package com.sd.business.service.order.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.file.utils.ObsFileUtil;
import com.ruoyi.common.annotation.LogicIgnore;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.core.domain.BaseSelectDto;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.system.service.ISysUserService;
import com.sd.business.entity.bom.bo.BomSpecBo;
import com.sd.business.entity.bom.po.Bom;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.in.dto.InOutStorageDto;
import com.sd.business.entity.in.emums.InDetailTypeEnum;
import com.sd.business.entity.in.emums.InOutTypeEnum;
import com.sd.business.entity.in.emums.OutDetailTypeEnum;
import com.sd.business.entity.in.po.InOutStorageBom;
import com.sd.business.entity.order.dto.OrderInfoDto;
import com.sd.business.entity.order.dto.OrderInfoSelectDto;
import com.sd.business.entity.order.dto.OrderPackageBomDto;
import com.sd.business.entity.order.dto.OrderSkuDto;
import com.sd.business.entity.order.dto.SkuSpecPackageBomDto;
import com.sd.business.entity.order.dto.SkuSpecPriceDto;
import com.sd.business.entity.order.enums.OrderClassifyEnum;
import com.sd.business.entity.order.enums.OrderExceptionTypeEnum;
import com.sd.business.entity.order.enums.OrderStatusEnum;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderPackageBom;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.order.po.OrderSkuBom;
import com.sd.business.entity.order.po.OrderSkuProductionCost;
import com.sd.business.entity.order.vo.OrderInfoVo;
import com.sd.business.entity.order.vo.OrderPackageBomVo;
import com.sd.business.entity.order.vo.OrderSkuBomVo;
import com.sd.business.entity.order.vo.OrderSkuExportVo;
import com.sd.business.entity.order.vo.OrderSkuVo;
import com.sd.business.entity.order.vo.SkuSpecPriceVo;
import com.sd.business.entity.price.po.PriceBillingStandard;
import com.sd.business.entity.price.po.PriceBillingStandardDetail;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.entity.sku.po.SkuSpecLink;
import com.sd.business.entity.statement.vo.StatementOrderClassifyTotalCountVo;
import com.sd.business.entity.warehouse.constant.WarehouseConstant;
import com.sd.business.entity.work.constant.MaterialsConstant;
import com.sd.business.entity.work.enums.WorkOrderFixationSpecEnum;
import com.sd.business.entity.work.po.WorkOrder;
import com.sd.business.entity.work.po.WorkOrderDetail;
import com.sd.business.mapper.order.OrderInfoMapper;
import com.sd.business.service.bom.BomService;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.department.DepartmentService;
import com.sd.business.service.in.InOutStorageService;
import com.sd.business.service.order.OrderInfoService;
import com.sd.business.service.order.OrderPackageBomService;
import com.sd.business.service.order.OrderSkuBomService;
import com.sd.business.service.order.OrderSkuProductionCostService;
import com.sd.business.service.order.OrderSkuService;
import com.sd.business.service.price.PriceBillingStandardDetailService;
import com.sd.business.service.price.PriceBillingStandardService;
import com.sd.business.service.sku.SkuSpecLinkService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.business.service.work.WorkOrderDetailService;
import com.sd.business.service.work.WorkOrderService;
import com.sd.business.service.work.impl.WorkOrderServiceImpl;
import com.sd.business.util.code.CodeEnum;
import com.sd.business.util.fixation.CoordinateBo;
import com.sd.business.util.fixation.FixationFactory;
import com.sd.business.util.fixation.FixationStrategy;
import com.sd.business.util.packing.GA;
import com.sd.business.util.packing.entity.PlaceItem;
import com.sd.business.util.packing.entity.Solution;
import com.sd.business.util.packing.model.Item;
import com.sd.framework.util.Assert;
import com.sd.framework.util.excel.util.ExcelUtil;
import com.sd.framework.util.sql.Sql;
import com.sd.framework.util.sql.Where;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * <p>
 * 订单 服务实现类
 * </p>
 *
 * @author
 * @since 2023-12-11
 */
@Service
public class OrderInfoServiceImpl extends ServiceImpl<OrderInfoMapper, OrderInfo> implements OrderInfoService {

    @Autowired
    private BomSpecService bomSpecService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private OrderSkuBomService orderSkuBomService;

    @Autowired
    private OrderPackageBomService orderPackageBomService;

    @Autowired
    private OrderSkuProductionCostService orderSkuProductionCostService;

    @Autowired
    private InOutStorageService inOutStorageService;

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private BomService bomService;

    @Autowired
    private DepartmentService departmentService;

    @Autowired
    private PriceBillingStandardService priceBillingStandardService;

    @Autowired
    private PriceBillingStandardDetailService priceBillingStandardDetailService;

    @Autowired
    private SkuSpecLinkService skuSpecLinkService;

    @Autowired
    private HttpServletResponse response;

    @Autowired
    private WorkOrderService workOrderService;

    @Autowired
    private WorkOrderDetailService workOrderDetailService;

    @Override
    public Page<OrderInfoVo> getPage(OrderInfoSelectDto dto) {

        Where<OrderInfoVo> sql = Sql.create(OrderInfoVo.class)
                .selectAll(OrderInfo.class)
                .selectAs(Department::getName, OrderInfoVo::getDepartmentName)

                .from(OrderInfo.class)
                .leftJoin(Department.class, OrderInfo::getDepartmentId)

                .orderByDesc("if(t1.status = " + OrderStatusEnum.SUSPEND.getKey() + ", 1, 0)")
                .orderByDesc(OrderInfo::getId)
                .eq(OrderInfo::getStatus, dto.getStatus())
                .eq(OrderInfo::getSettlementStatus, dto.getSettlementStatus())
                .eq(OrderInfo::getFlowStatus, dto.getFlowStatus())
                .like(OrderInfo::getCode, dto.getCode())
                .like(OrderInfo::getWlnCode, dto.getWlnCode())
                .ge(OrderInfo::getWlnCreateTime, dto.getBeginTime())
                .le(OrderInfo::getWlnCreateTime, dto.getEndTime())
                .findInSet(OrderInfo::getExceptionType, dto.getExceptionType())
                .eq(Department::getId, dto.getDepartmentId())
                .like(Department::getName, dto.getDepartmentName());

        List<Long> detpIdList = sysUserService.getDetpIdListByUserId(SecurityUtils.getUserId());
        if (!detpIdList.isEmpty()) {
            sql.in(OrderInfo::getDepartmentId, detpIdList);
            sql.eq(OrderInfo::getCreateUser, SecurityUtils.getUserId());
        }

        if (StatusConstant.YES.equals(dto.getLinkedStatementOfAccount())) {
            sql.isNotNull(OrderInfo::getStatementOfAccountId);
            sql.in(OrderInfo::getStatus, OrderStatusEnum.COMPLETION_PRODUCTION.getKey(), OrderStatusEnum.HAVE_BEEN_SHIPPED.getKey());
        } else if (StatusConstant.NO.equals(dto.getLinkedStatementOfAccount())) {
            sql.isNull(OrderInfo::getStatementOfAccountId);
            sql.in(OrderInfo::getStatus, OrderStatusEnum.COMPLETION_PRODUCTION.getKey(), OrderStatusEnum.HAVE_BEEN_SHIPPED.getKey());
        }

        if (StatusConstant.YES.equals(dto.getException())) {
            sql.ne(OrderInfo::getExceptionType, StatusConstant.NO);
        } else if (StatusConstant.NO.equals(dto.getException())) {
            sql.eq(OrderInfo::getExceptionType, StatusConstant.NO);
        }

        Page<OrderInfoVo> page = sql.page(dto);
        List<OrderInfoVo> records = page.getRecords();

        if (records.isEmpty()) {
            return page;
        }

        for (OrderInfoVo record : records) {
            // 设置异常提示
            String exceptionType = record.getExceptionType();
            if (exceptionType.equals("0")) {
                continue;
            }
            String exceptionTypeDetail = Arrays.stream(exceptionType.split(","))
                    .map(Convert::toInt)
                    .map(OrderExceptionTypeEnum::getEnumValue)
                    .collect(Collectors.joining(","));
            record.setExceptionTypeDetail(exceptionTypeDetail);
        }

        return page;
    }

    @Override
    public OrderInfoVo detail(Long id) {

        OrderInfoVo vo = Sql.create(OrderInfoVo.class)
                .selectAll(OrderInfo.class)
                .selectAs(Department::getName, OrderInfoVo::getDepartmentName)
                .from(OrderInfo.class)
                .leftJoin(Department.class, OrderInfo::getDepartmentId)
                .eq(OrderInfo::getId, id)
                .one();

        Assert.notNull(vo, "未知订单");

        // 订单sku列表
        List<OrderSkuVo> orderSkuList = Sql.create(OrderSkuVo.class)
                .selectAll(OrderSku.class)
                .selectAs(SkuSpec::getCode, OrderSkuVo::getSkuSpecCode)
                .selectAs(SkuSpec::getName, OrderSkuVo::getSkuSpecName)
                .selectAs(BomSpec::getCode, OrderSkuVo::getBomSpecCode)
                .selectAs(BomSpec::getName, OrderSkuVo::getBomSpecName)
                .from(OrderSku.class)
                .leftJoin(SkuSpec.class, OrderSku::getSkuSpecId)
                .leftJoin(BomSpec.class, OrderSku::getBomSpecId)
                .eq(OrderSku::getOrderId, id)
                .list();

        // 包材配件/单品
        List<OrderSkuBomVo> orderSkuBomList = Sql.create(OrderSkuBomVo.class)
                .selectAll(OrderSkuBom.class)
                .selectAs(BomSpec::getCode, OrderSkuBomVo::getBomSpecCode)
                .selectAs(BomSpec::getName, OrderSkuBomVo::getBomSpecName)
                .selectAs(Bom::getBomClassifyId, OrderSkuBomVo::getBomClassifyId)
                .from(OrderSkuBom.class)
                .leftJoin(BomSpec.class, OrderSkuBom::getBomSpecId)
                .leftJoin(Bom.class, BomSpec::getBomId)
                .eq(OrderSkuBom::getOrderId, id)
                .list();

        // 包装明细
        List<OrderPackageBomVo> orderPackageBomList = Sql.create(OrderPackageBomVo.class)
                .selectAll(OrderPackageBom.class)
                .selectAs(BomSpec::getCode, OrderPackageBomVo::getBomSpecCode)
                .selectAs(BomSpec::getName, OrderPackageBomVo::getBomSpecName)
                .from(OrderPackageBom.class)
                .leftJoin(BomSpec.class, OrderPackageBom::getBomSpecId)
                .eq(OrderPackageBom::getOrderId, id)
                .list();

        // 赋值
        Map<Long, List<OrderSkuBomVo>> map = orderSkuBomList.stream().collect(Collectors.groupingBy(OrderSkuBomVo::getOrderSkuId));
        orderSkuList.forEach(item -> item.setOrderSkuBomList(map.getOrDefault(item.getId(), Collections.emptyList())));
        vo.setOrderSkuList(orderSkuList);
        vo.setOrderPackageBomList(orderPackageBomList);

        return vo;
    }

    @DSTransactional
    @Override
    public void add(OrderInfoDto dto) {

        // 新增订单
        dto.setId(IdWorker.getId());
        dto.setSource(1);
        dto.setCode("PI" + new Date().getTime());
        dto.setSettlementStatus(1);
        dto.setTaxRate(BigDecimal.TEN);
        dto.setTag("0");
        dto.setExceptionType(OrderExceptionTypeEnum.NORMAL.getKey().toString());
        dto.setClassify(OrderClassifyEnum.PURCHASE_ORDER.getKey());

        // 保存订单sku列表
        List<OrderSku> orderSkuList = getOrderSkuList(dto, true);
        orderSkuService.saveBatch(orderSkuList);

        // 保存订单sku包材信息
        List<OrderSkuBom> orderSkuBomList = getOrderSkuBomList(dto);
        orderSkuBomService.saveBatch(orderSkuBomList);

        // 保存订单包装信息
        List<OrderPackageBom> orderPackageBomList = getOrderPackageBomList(dto);
        if (ObjectUtil.isNotEmpty(orderPackageBomList)) {
            orderPackageBomService.saveBatch(orderPackageBomList);
        }

        // 新增订单产品生产成本
        List<OrderSkuProductionCost> orderSkuProductionCostList = getOrderSkuProductionCostList(orderSkuList, orderSkuBomList);
        orderSkuProductionCostService.saveBatch(orderSkuProductionCostList);

        // 保存订单
        save(dto);

        // 保存附件
        ObsFileUtil.saveFile(dto.getFileList(), dto.getId());

        // 保存产品不干胶图稿
        for (OrderSkuDto item : dto.getOrderSkuList()) {
            ObsFileUtil.saveFile(item.getSelfAdhesiveStickerFile(), item.getId());
        }

    }

    @DSTransactional
    @Override
    public void edit(OrderInfoDto dto) {

        // 更新订单sku列表
        List<OrderSku> orderSkuList = getOrderSkuList(dto, false);
        orderSkuService.editLinked(orderSkuList, OrderSku::getOrderId, dto.getId());

        // 更新订单sku包材信息
        List<OrderSkuBom> orderSkuBomList = getOrderSkuBomList(dto);
        orderSkuBomService.editLinked(orderSkuBomList, OrderSkuBom::getOrderId, dto.getId());

        // 更新订单包装信息
        List<OrderPackageBom> orderPackageBomList = getOrderPackageBomList(dto);
        orderPackageBomService.editLinked(orderPackageBomList, OrderPackageBom::getOrderId, dto.getId());

        // 更新订单产品生产成本
        List<OrderSkuProductionCost> orderSkuProductionCostList = getOrderSkuProductionCostList(orderSkuList, orderSkuBomList);
        orderSkuProductionCostService.remove(q -> q.eq(OrderSkuProductionCost::getOrderId, dto.getId()));
        orderSkuProductionCostService.saveBatch(orderSkuProductionCostList);

        // 更新订单
        updateById(dto);

        // 更新订单附件
        ObsFileUtil.editFile(dto.getFileList(), dto.getId());

        // 更新订单产品不干胶图稿
        for (OrderSkuDto item : dto.getOrderSkuList()) {
            ObsFileUtil.editFile(item.getSelfAdhesiveStickerFile(), item.getId());
        }

    }

    @DSTransactional
    @Override
    public void delete(Long id) {

        boolean b = WorkOrderServiceImpl.schedulingLock.tryLock();
        if (!b) {
            throw new ServiceException("正在排版中，无法删除订单");
        }

        try {
            OrderInfo orderInfo = getById(id);

            if (orderInfo == null) {
                throw new ServiceException("未找到订单");
            }

            if (Objects.equals(orderInfo.getDelFlag(), StatusConstant.NOT_DELETED)
                    && orderInfo.getStatus() > OrderStatusEnum.SCHEDULING.getKey()) {
                throw new ServiceException("订单已排单生产，无法删除");
            }

            // 删除订单重新排单
            reschedule(id);

            List<Long> orderSkuIdList = orderSkuService.listObject(BaseIdPo::getId, q -> q.eq(OrderSku::getOrderId, id));

            removeById(id);
            orderSkuService.remove(q -> q.eq(OrderSku::getOrderId, id));
            orderSkuBomService.remove(q -> q.eq(OrderSkuBom::getOrderId, id));
            orderPackageBomService.remove(q -> q.eq(OrderPackageBom::getOrderId, id));
            orderSkuProductionCostService.remove(q -> q.eq(OrderSkuProductionCost::getOrderId, id));
            ObsFileUtil.removeFile(id);
            ObsFileUtil.removeFile(orderSkuIdList);

        } catch (ServiceException e) {
            throw e;
        } catch (Exception e) {
            log.error("删除订单失败", e);
            throw new ServiceException("删除订单失败");
        } finally {
            WorkOrderServiceImpl.schedulingLock.unlock();
        }
    }

    @DSTransactional
    @Override
    public void confirmation(OrderInfoDto dto) {
        OrderInfo order = getById(dto.getId());

        if (ObjectUtil.notEqual(order.getStatus(), OrderStatusEnum.DRAFT.getKey())
                && ObjectUtil.notEqual(order.getStatus(), OrderStatusEnum.UNDER_REVIEW.getKey())) {
            throw new ServiceException("订单不在草稿或待确认状态");
        }

        dto.setStatus(OrderStatusEnum.SCHEDULING.getKey());
        edit(dto);

    }

    @Override
    public SkuSpecPriceVo getSkuSpecPrice(SkuSpecPriceDto dto) {
        // 查询单价
        Long skuSpecId = dto.getSkuSpecId();
        SkuSpec skuSpec = skuSpecService.getById(skuSpecId);
        Assert.notNull(skuSpec, "未知sku规格");
        Long bomSpecId = skuSpec.getBomSpecId();
        Assert.notNull(bomSpecId, "sku未绑定bom");
        BomSpec bomSpec = bomSpecService.getById(bomSpecId);
        Assert.notNull(bomSpec, "未知bom规格");
        Bom bom = bomService.getById(bomSpec.getBomId());
        Assert.notNull(bom, "未知bom");

        // 其他价格查询
        Long deptId = SecurityUtils.getDeptId();
        Department department = departmentService.getById(deptId);
        Assert.notNull(department, "未知事业部");
        String priceBillingStandardIdStr = department.getPriceBillingStandardId();
        Assert.notBlank(priceBillingStandardIdStr, "事业部未绑定加工计费标准");

        List<String> priceBillingStandardIdList = Arrays.stream(priceBillingStandardIdStr.split(",")).collect(Collectors.toList());
        List<PriceBillingStandard> priceBillingStandardList = priceBillingStandardService.listByIds(priceBillingStandardIdList);

        List<PriceBillingStandardDetail> priceBillingStandardDetailList = Collections.emptyList();

        if (StrUtil.isNotBlank(bom.getSpecies())) {
            for (PriceBillingStandard priceBillingStandard : priceBillingStandardList) {
                if (Objects.equals(bom.getSpecies(), priceBillingStandard.getSpecies())) {
                    priceBillingStandardDetailList = priceBillingStandardDetailService.list(
                            q -> q.eq(PriceBillingStandardDetail::getPriceBillingStandardId, priceBillingStandard.getId()));
                    break;
                }
            }
        }

        // 赋值
        SkuSpecPriceVo skuSpecPriceVo = new SkuSpecPriceVo();
        skuSpecPriceVo.setUnitPrice(bomSpec.getInternalSellingPrice());
        skuSpecPriceVo.setCustomProcessingFee(BigDecimal.ZERO);
        skuSpecPriceVo.setLssueFee(BigDecimal.ZERO);
        skuSpecPriceVo.setDeliveryMaterialsFee(BigDecimal.ZERO);
        skuSpecPriceVo.setPackingLabor(BigDecimal.ZERO);
        skuSpecPriceVo.setManagementFee(BigDecimal.ZERO);

        // TODO 根据收费项目赋值价格
        for (PriceBillingStandardDetail priceBillingStandardDetail : priceBillingStandardDetailList) {
            String chargeItem = priceBillingStandardDetail.getChargeItem();
            BigDecimal chargePrice = ObjectUtil.defaultIfNull(priceBillingStandardDetail.getChargePrice(), BigDecimal.ZERO);
            switch (chargeItem) {
                case "40":
                    skuSpecPriceVo.setPackingLabor(chargePrice);
                    break;
                case "50":
                    skuSpecPriceVo.setLssueFee(chargePrice);
                    break;
                case "60":
                    skuSpecPriceVo.setDeliveryMaterialsFee(chargePrice);
                    break;
                case "70":
                    skuSpecPriceVo.setManagementFee(chargePrice);
                    break;
                default:
                    //if (Objects.equals(chargeItem, skuSpec.getMachinedPanel())
                    //        && dto.getQuantity().compareTo(priceBillingStandardDetail.getSectionMin()) >= 0
                    //        && dto.getQuantity().compareTo(priceBillingStandardDetail.getSectionMax()) < 0) {
                    //    skuSpecPriceVo.setCustomProcessingFee(chargePrice);
                    //    skuSpecPriceVo.setCustomProcessingType(chargeItem);
                    //}
            }
        }

        return skuSpecPriceVo;
    }

    @Override
    public void turnToRegularOrder(Long id) {
        OrderInfo orderInfo = getById(id);
        String exceptionType = orderInfo.getExceptionType();

        if (Objects.equals(exceptionType, OrderExceptionTypeEnum.NORMAL.getKey().toString())) {
            return;
        }

        for (String s : exceptionType.split(",")) {
            OrderExceptionTypeEnum orderExceptionTypeEnum = OrderExceptionTypeEnum.getEnum(Convert.toInt(s));
            if (ObjectUtil.notEqual(orderExceptionTypeEnum, OrderExceptionTypeEnum.ORDER_CLOSURE)
                    && ObjectUtil.notEqual(orderExceptionTypeEnum, OrderExceptionTypeEnum.ORDER_ABORT)
                    && ObjectUtil.notEqual(orderExceptionTypeEnum, OrderExceptionTypeEnum.ORDER_EXCEPTION_HANDLING)) {
                throw new ServiceException(orderExceptionTypeEnum.getValue() + "，无法转为正常订单");
            }
        }

        update(q -> q
                .eq(BaseIdPo::getId, orderInfo.getId())
                .set(OrderInfo::getExceptionType, OrderExceptionTypeEnum.NORMAL.getKey().toString())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );

        // 转trade
        if (Arrays.asList(exceptionType.split(",")).contains(OrderExceptionTypeEnum.ORDER_CLOSURE.getKey().toString())) {
            // 订单分类修改为无理由订单
            update(q -> q
                    .eq(BaseIdPo::getId, orderInfo.getId())
                    .set(OrderInfo::getClassify, OrderClassifyEnum.NO_REASON_ORDER.getKey())
            );
        }
    }

    @Override
    public String getOrderDeliveryCode() {
        return CodeEnum.SH_CODE.getCode();
    }

    @Override
    public void saveOrderDeliveryCode(OrderInfoDto dto) {
        OrderInfo orderInfo = getById(dto.getId());
        Assert.notNull(orderInfo, "未找到订单");
        Assert.notBlank(dto.getDeliveryCode(), "送货单号不能为空");
        Long count = count(q -> q.eq(OrderInfo::getDeliveryCode, dto.getDeliveryCode()).last("limit 1"));
        Assert.eqZero(count, "送货单号重复，请重新生成！");
        update(q -> q.set(OrderInfo::getDeliveryCode, dto.getDeliveryCode()).eq(BaseIdPo::getId, dto.getId()));
    }

    @Override
    public void suspendOrder(Long id) {
        OrderInfo orderInfo = getById(id);
        Assert.notNull(orderInfo, "未找到订单");
        update(q -> q.set(OrderInfo::getStatus, OrderStatusEnum.SUSPEND.getKey()).eq(BaseIdPo::getId, id));
    }

    @Override
    public void cancelSuspendOrder(Long id) {
        OrderInfo orderInfo = getById(id);
        Assert.notNull(orderInfo, "未找到订单");
        update(q -> q.set(OrderInfo::getStatus, OrderStatusEnum.SCHEDULING.getKey()).eq(BaseIdPo::getId, id));
    }

    @Override
    public void editOrderPackageBom(OrderInfoDto dto) {
        OrderInfo orderInfo = getById(dto.getId());
        Assert.notNull(orderInfo, "未找到订单");

        // 生产中和生产完成的订单快递包材归还入库
        if (ObjectUtil.equals(orderInfo.getStatus(), OrderStatusEnum.IN_PRODUCTION.getKey())
                || ObjectUtil.equals(orderInfo.getStatus(), OrderStatusEnum.COMPLETION_PRODUCTION.getKey())) {
            dto.setCode(orderInfo.getCode());
            dto.setWlnCode(orderInfo.getWlnCode());
            orderPackageBomInOutStorage(dto);
        }

        List<OrderPackageBomDto> tempOrderPackageBomList = dto.getOrderPackageBomList();

        BigDecimal newDeliveryMaterialsFee = BigDecimal.ZERO;
        if (ObjectUtil.isEmpty(tempOrderPackageBomList)) {
            orderPackageBomService.remove(q -> q.eq(OrderPackageBom::getOrderId, orderInfo.getId()));
        } else {
            List<OrderPackageBom> orderPackageBomList = tempOrderPackageBomList.stream()
                    .peek(item -> item.setOrderId(dto.getId()))
                    .map(item -> (OrderPackageBom) item)
                    .collect(Collectors.toList());
            orderPackageBomService.editLinked(orderPackageBomList, OrderPackageBom::getOrderId, orderInfo.getId());
            newDeliveryMaterialsFee = orderPackageBomList.stream()
                    .map(item -> item.getInternalSellingPrice().multiply(item.getQuantity()))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
        }
        orderInfo.setDeliveryMaterialsFee(newDeliveryMaterialsFee);

        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, orderInfo.getId()));
        BigDecimal quantity = orderSkuList.stream().map(OrderSku::getQuantity).reduce(BigDecimal.ZERO, BigDecimal::add);
        BigDecimal totalAmount = orderInfo.getTotalAmount();
        BigDecimal oldDeliveryMaterialsFee = orderInfo.getDeliveryMaterialsFee();
        orderSkuList.forEach(item -> item.setDeliveryMaterialsFee(
                orderInfo.getDeliveryMaterialsFee().divide(quantity, 4, RoundingMode.HALF_UP)));
        orderInfo.setTotalAmount(totalAmount.subtract(oldDeliveryMaterialsFee).add(newDeliveryMaterialsFee));
        updateById(orderInfo);
        orderSkuService.updateBatchById(orderSkuList);
        ObsFileUtil.editFile(dto.getOuterBoxSelfAdhesiveStickerFile(), dto.getId(), 1);
    }

    @Override
    public List<OrderPackageBomVo> getSkuSpecPackageBomList(SkuSpecPackageBomDto dto) {
        List<SkuSpecPriceDto> skuSpecList = dto.getSkuSpecList();
        List<Long> skuSpecIds = skuSpecList.stream().map(SkuSpecPriceDto::getSkuSpecId).collect(Collectors.toList());
        Map<Long, List<SkuSpecLink>> skuSpecMap = skuSpecLinkService.mapKGroup(
                SkuSpecLink::getSkuSpecId,
                q -> q.in(SkuSpecLink::getSkuSpecId, skuSpecIds).eq(SkuSpecLink::getType, 2));
        if (ObjectUtil.isEmpty(skuSpecMap)) {
            return Collections.emptyList();
        }
        List<Long> bomSpecIds = skuSpecMap.values().stream()
                .flatMap(item -> item.stream().map(SkuSpecLink::getBomSpecId))
                .collect(Collectors.toList());
        Map<Long, BomSpec> bomSpecPriceMap = bomSpecService.mapKEntity(BaseIdPo::getId, q -> q.in(BaseIdPo::getId, bomSpecIds));

        List<OrderPackageBomVo> list = new ArrayList<>();
        for (SkuSpecPriceDto skuSpecPriceDto : skuSpecList) {
            List<SkuSpecLink> skuSpecLinkList = skuSpecMap.getOrDefault(skuSpecPriceDto.getSkuSpecId(), Collections.emptyList());
            List<OrderPackageBomVo> packageBomVoList = skuSpecLinkList.stream().map(item -> {
                BomSpec bomSpec = bomSpecPriceMap.get(item.getBomSpecId());
                OrderPackageBomVo orderPackageBomVo = new OrderPackageBomVo();
                orderPackageBomVo.setBomSpecName(bomSpec.getName());
                orderPackageBomVo.setBomSpecCode(bomSpec.getCode());
                orderPackageBomVo.setCostPrice(bomSpec.getCostPrice());
                orderPackageBomVo.setInternalSellingPrice(bomSpec.getInternalSellingPrice());
                orderPackageBomVo.setBomSpecId(item.getBomSpecId());
                orderPackageBomVo.setQuantity(item.getQuantity().multiply(skuSpecPriceDto.getQuantity()));
                return orderPackageBomVo;
            }).collect(Collectors.toList());
            list.addAll(packageBomVoList);
        }
        // 合并相同bom
        Map<Long, OrderPackageBomVo> bomVoMap = list.stream().collect(Collectors.toMap(
                OrderPackageBom::getBomSpecId,
                Function.identity(),
                (v1, v2) -> {
                    v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                    return v1;
                }));
        return new ArrayList<>(bomVoMap.values());
    }

    @LogicIgnore
    @Override
    public Page<OrderInfoVo> deletedOrderPage(OrderInfoSelectDto dto) {
        return Sql.create(OrderInfoVo.class)
                .selectAll(OrderInfo.class)
                .selectAs(Department::getName, OrderInfoVo::getDepartmentName)
                .from(OrderInfo.class)
                .leftJoin(Department.class, OrderInfo::getDepartmentId)
                .orderByDesc(OrderInfo::getId)
                .eq(Department::getId, dto.getDepartmentId())
                .like(OrderInfo::getCode, dto.getCode())
                .like(OrderInfo::getWlnCode, dto.getWlnCode())
                .eq(OrderInfo::getDelFlag, StatusConstant.DELETED)
                .page(dto);
    }

    @LogicIgnore
    @Override
    public OrderInfoVo deletedOrderDetail(BaseSelectDto dto) {
        return detail(dto.getId());
    }

    @Override
    public void orderExportExcel(Long id) {
        OrderInfo orderInfo = this.getById(id);
        List<OrderSku> list = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, id));

        List<Long> bomSpecIds = list.stream().map(OrderSku::getBomSpecId).collect(Collectors.toList());
        Map<Long, BomSpecBo> bomSpecBoMap = bomSpecService.getBomSpecBo(bomSpecIds);

        List<Long> skuSpecIds = list.stream().map(OrderSku::getSkuSpecId).collect(Collectors.toList());
        Map<Long, SkuSpec> skuSpecMap = skuSpecService.byIdsToMap(skuSpecIds);

        List<OrderSku> orderPackageSkuList = list.stream()
                .filter(item -> !Objects.equals(bomSpecBoMap.get(item.getBomSpecId()).getClassifyParentId(), 1L))
                .collect(Collectors.toList());

        List<OrderSku> orderSkuList = list.stream()
                .filter(item -> Objects.equals(bomSpecBoMap.get(item.getBomSpecId()).getClassifyParentId(), 1L))
                .collect(Collectors.toList());

        BigDecimal orderPackageSku = orderPackageSkuList.stream()
                .map(item -> item.getUnitPrice().add(item.getDeliveryMaterialsFee()).add(item.getPackagingMaterialCost()).multiply(item.getQuantity()))
                .reduce(BigDecimal.ZERO, BigDecimal::add);

        BigDecimal orderPackagePrice = orderPackageSku.divide(BigDecimal.valueOf(orderSkuList.size()), 2, RoundingMode.HALF_UP);

        List<OrderSkuExportVo> exportVoList = orderSkuList.stream().map(item -> {
            BigDecimal subtotal = item.getUnitPrice()
                    .add(item.getDeliveryMaterialsFee())
                    .add(item.getPackagingMaterialCost());
            BomSpecBo bomSpecBo = bomSpecBoMap.get(item.getBomSpecId());
            SkuSpec skuSpec = skuSpecMap.get(item.getSkuSpecId());
            OrderSkuExportVo orderSkuExportVo = new OrderSkuExportVo();
            orderSkuExportVo.setWlnCode(orderInfo.getWlnCode());
            orderSkuExportVo.setSkuSpecCode(skuSpec.getCode());
            orderSkuExportVo.setBomSpecCode(bomSpecBo.getBomSpecCode());
            orderSkuExportVo.setErpCode(item.getErpCode());
            orderSkuExportVo.setFeatureCode(item.getFeatureCode());
            orderSkuExportVo.setQuantity(item.getQuantity());
            orderSkuExportVo.setTotalAmount(subtotal.multiply(item.getQuantity()).add(orderPackagePrice));
            return orderSkuExportVo;
        }).collect(Collectors.toList());

        ExcelUtil.export(response, exportVoList, OrderSkuExportVo.class);
    }

    @Override
    public StatementOrderClassifyTotalCountVo getOrderClassifyTotalCount(List<Long> idList) {
        Assert.notEmpty(idList, "对账单id不能为空");
        IWrapper<OrderInfo> wrapper = getWrapper();
        wrapper.in("o", OrderInfo::getStatementOfAccountId, idList);
        return this.baseMapper.getOrderClassifyTotalCount(wrapper);
    }

    /**
     * 订单修改快递包材出入库操作
     */
    private void orderPackageBomInOutStorage(OrderInfoDto dto) {
        List<OrderPackageBom> oldOrderPackageBomList = orderPackageBomService.list(q -> q.eq(OrderPackageBom::getOrderId, dto.getId()));
        List<OrderPackageBomDto> newOrderPackageBomList = dto.getOrderPackageBomList();

        // 旧bom
        Map<Long, BigDecimal> oldBomMap = oldOrderPackageBomList.stream()
                .collect(Collectors.toMap(
                        OrderPackageBom::getBomSpecId,
                        OrderPackageBom::getQuantity,
                        BigDecimal::add
                ));
        Map<Long, BigDecimal> newBomMap = newOrderPackageBomList.stream()
                .collect(Collectors.toMap(
                        OrderPackageBom::getBomSpecId,
                        OrderPackageBom::getQuantity,
                        BigDecimal::add
                ));
        // 查询所有bom进行对比
        Set<Long> bomIds = new HashSet<>(oldBomMap.keySet());
        bomIds.addAll(newBomMap.keySet());
        Map<Long, BigDecimal> outBomMap = new HashMap<>();
        Map<Long, BigDecimal> inBomMap = new HashMap<>();
        for (Long bomId : bomIds) {
            BigDecimal oldReturnQuantity = oldBomMap.get(bomId);
            BigDecimal newReturnQuantity = newBomMap.get(bomId);
            // 旧bom没有，出库库存，反之则归还库存
            if (oldReturnQuantity == null) {
                outBomMap.put(bomId, newReturnQuantity);
                continue;
            } else if (newReturnQuantity == null) {
                inBomMap.put(bomId, oldReturnQuantity);
                continue;
            }
            // 都有值，判断数量
            if (oldReturnQuantity.compareTo(newReturnQuantity) == 0) {
                continue;
            }
            // 旧bom大于新bom则归还入库，反之出库库存
            if (oldReturnQuantity.compareTo(newReturnQuantity) > 0) {
                inBomMap.put(bomId, oldReturnQuantity.subtract(newReturnQuantity));
            } else {
                outBomMap.put(bomId, newReturnQuantity.subtract(oldReturnQuantity));
            }
        }
        if (!outBomMap.isEmpty()) {
            // 包材从包材仓出库
            List<InOutStorageBom> outStorageBomList = outBomMap.entrySet().stream()
                    .map(item -> {
                        InOutStorageBom inOutStorageBom = new InOutStorageBom();
                        inOutStorageBom.setBomSpecId(item.getKey());
                        inOutStorageBom.setQuantity(item.getValue());
                        return inOutStorageBom;
                    })
                    .collect(Collectors.toList());
            InOutStorageDto inOutStorageDto = new InOutStorageDto();
            inOutStorageDto.setType(InOutTypeEnum.OUT.getKey());
            inOutStorageDto.setDetailType(OutDetailTypeEnum.PRODUCTION.getKey());
            inOutStorageDto.setWarehouseId(WarehouseConstant.PACKAGING_MATERIAL);
            inOutStorageDto.setApplicant(SecurityUtils.getLoginUser().getUser().getNickName());
            inOutStorageDto.setInOutStorageBomList(outStorageBomList);
            inOutStorageService.add(inOutStorageDto);
        }
        if (!inBomMap.isEmpty()) {
            // 包材归还入库
            List<InOutStorageBom> inOutStorageBomList = inBomMap.entrySet()
                    .stream()
                    .map(item -> {
                        InOutStorageBom inOutStorageBom = new InOutStorageBom();
                        inOutStorageBom.setBomSpecId(item.getKey());
                        inOutStorageBom.setQuantity(item.getValue());
                        return inOutStorageBom;
                    })
                    .collect(Collectors.toList());
            InOutStorageDto packagingMaterialInOutStorageDto = new InOutStorageDto();
            packagingMaterialInOutStorageDto.setType(InOutTypeEnum.IN.getKey());
            packagingMaterialInOutStorageDto.setDetailType(InDetailTypeEnum.RETURN_GOODS.getKey());
            packagingMaterialInOutStorageDto.setWarehouseId(WarehouseConstant.PACKAGING_MATERIAL);
            packagingMaterialInOutStorageDto.setApplicant(SecurityUtils.getLoginUser().getUser().getNickName());
            packagingMaterialInOutStorageDto.setRemark("订单产品快递包装修改：" + dto.getCode()
                    + (StrUtil.isBlank(dto.getWlnCode()) ? StringPool.EMPTY : "（" + dto.getWlnCode() + "）")
                    + " 包材归还入库");
            packagingMaterialInOutStorageDto.setInOutStorageBomList(inOutStorageBomList);
            inOutStorageService.add(packagingMaterialInOutStorageDto);
        }
    }

    /**
     * 获取订单sku列表
     */
    private List<OrderSku> getOrderSkuList(OrderInfoDto dto, boolean isAdd) {
        return dto.getOrderSkuList().stream()
                .peek(item -> {
                    if (isAdd) {
                        item.setSchedulingNum(BigDecimal.ZERO);
                        item.setCompleteScheduling(StatusConstant.NO);
                    }
                })
                .peek(item -> item.setOrderId(dto.getId()))
                .map(item -> (OrderSku) item)
                .collect(Collectors.toList());
    }

    /**
     * 获取订单sku包材列表
     */
    private List<OrderSkuBom> getOrderSkuBomList(OrderInfoDto dto) {
        return dto.getOrderSkuList().stream()
                .flatMap(orderSku -> orderSku.getOrderSkuBomList().stream()
                        .peek(orderSkuBom -> orderSkuBom.setOrderId(dto.getId()))
                        .peek(orderSkuBom -> orderSkuBom.setOrderSkuId(orderSku.getId())))
                .collect(Collectors.toList());
    }

    /**
     * 获取订单包装列表
     */
    private List<OrderPackageBom> getOrderPackageBomList(OrderInfoDto dto) {
        if (ObjectUtil.isEmpty(dto.getOrderPackageBomList())) {
            return Collections.emptyList();
        }
        return dto.getOrderPackageBomList().stream()
                .peek(item -> item.setOrderId(dto.getId()))
                .map(item -> (OrderPackageBom) item)
                .collect(Collectors.toList());
    }

    /**
     * 获取订单sku生产成本列表
     */
    private List<OrderSkuProductionCost> getOrderSkuProductionCostList(List<OrderSku> orderSkuList, List<OrderSkuBom> orderSkuBomList) {
        Map<Long, List<OrderSkuBom>> orderSkuBomMap = orderSkuBomList.stream().collect(Collectors.groupingBy(OrderSkuBom::getOrderSkuId));
        return orderSkuList.stream().map(item -> {
            List<OrderSkuBom> bomList = orderSkuBomMap.getOrDefault(item.getId(), Collections.emptyList());
            BigDecimal productPackagingMaterialCost = bomList.stream()
                    .map(orderSkuBom -> orderSkuBom.getQuantity().multiply(orderSkuBom.getUnitPrice()).multiply(item.getQuantity()))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);

            OrderSkuProductionCost orderSkuProductionCost = new OrderSkuProductionCost();
            orderSkuProductionCost.setOrderId(item.getOrderId());
            orderSkuProductionCost.setOrderSkuId(item.getId());
            // 材料成本
            orderSkuProductionCost.setMaterialCost(item.getUnitPrice().multiply(item.getQuantity()));
            // 辅料成本暂时没有默认为零
            orderSkuProductionCost.setAuxiliaryMaterialCost(BigDecimal.ZERO);
            // 产品包材成本
            orderSkuProductionCost.setProductPackagingMaterialCost(productPackagingMaterialCost);
            // 物流包材成本
            orderSkuProductionCost.setLogisticsPackagingMaterialCost(item.getDeliveryMaterialsFee().multiply(item.getQuantity()));
            // 成本总
            orderSkuProductionCost.setTotalAmount(orderSkuProductionCost.getMaterialCost()
                    .add(orderSkuProductionCost.getAuxiliaryMaterialCost())
                    .add(orderSkuProductionCost.getProductPackagingMaterialCost())
                    .add(orderSkuProductionCost.getLogisticsPackagingMaterialCost()));
            return orderSkuProductionCost;
        }).collect(Collectors.toList());
    }

    /**
     * 重新排单
     */
    private void reschedule(Long id) {
        // 获取所有已排订单
        List<WorkOrderDetail> list = workOrderDetailService.list(q -> q.eq(WorkOrderDetail::getOrderInfoId, id));
        if (list.isEmpty()) {
            return;
        }
        workOrderDetailService.remove(q -> q.eq(WorkOrderDetail::getOrderInfoId, id));

        Set<Long> workOrderIdSet = list.stream().map(WorkOrderDetail::getWorkOrderId).collect(Collectors.toSet());
        List<WorkOrder> workOrderList = workOrderService.list(q -> q.in(BaseIdPo::getId, workOrderIdSet));

        List<WorkOrderDetail> workOrderDetailList = workOrderDetailService.list(q -> q.in(WorkOrderDetail::getWorkOrderId, workOrderIdSet));
        Map<Long, List<WorkOrderDetail>> map = workOrderDetailList.stream().collect(Collectors.groupingBy(WorkOrderDetail::getWorkOrderId));

        for (WorkOrder workOrder : workOrderList) {
            List<WorkOrderDetail> tempWorkOrderDetailList = map.get(workOrder.getId());
            if (ObjectUtil.isEmpty(tempWorkOrderDetailList)) {
                workOrder.setSchedulingNum(BigDecimal.ZERO);
                workOrder.setUsedArea(BigDecimal.ZERO);
                workOrder.setUseRatio(BigDecimal.ZERO);
                workOrder.setMaxLength(BigDecimal.ZERO);
                workOrderService.updateById(workOrder);
                continue;
            }

            // 固定尺寸排版
            if (Objects.equals(workOrder.getType(), 1)) {

                String[] split = workOrder.getFixationSpec().split("\\*");
                BigDecimal length = new BigDecimal(split[0]);
                BigDecimal width = new BigDecimal(split[1]);

                WorkOrderFixationSpecEnum workOrderFixationSpecEnum = WorkOrderFixationSpecEnum.getEnum(length, width);

                FixationStrategy fixationStrategy = FixationFactory.get(workOrderFixationSpecEnum);

                for (WorkOrderDetail orderDetail : tempWorkOrderDetailList) {
                    CoordinateBo coordinateBo = fixationStrategy.next();
                    orderDetail.setLength(coordinateBo.getLength());
                    orderDetail.setWidth(coordinateBo.getWidth());
                    orderDetail.setBleedingLength(coordinateBo.getBleedingLength());
                    orderDetail.setBleedingWidth(coordinateBo.getBleedingWidth());
                    orderDetail.setRotate(coordinateBo.getHasRotate() ? StatusConstant.YES : StatusConstant.NO);
                    orderDetail.setX(coordinateBo.getX());
                    orderDetail.setY(coordinateBo.getY());
                }

                workOrder.setSchedulingNum(fixationStrategy.getSchedulingNum());
                workOrder.setUsedArea(fixationStrategy.getUsedArea());
                workOrder.setUseRatio(fixationStrategy.getUseRatio());
                workOrder.setMaxLength(fixationStrategy.getMaxLength());
            }

            // 非固定尺寸排版
            else {

                Item[] items = tempWorkOrderDetailList.stream().map(item -> {
                    boolean rotate = Objects.equals(item.getRotate(), StatusConstant.YES);
                    BigDecimal bleedingLength = rotate ? item.getBleedingWidth() : item.getBleedingLength();
                    BigDecimal bleedingWidth = rotate ? item.getBleedingLength() : item.getBleedingWidth();

                    return new Item(item.getId(), bleedingLength.doubleValue(), bleedingWidth.doubleValue());
                }).toArray(Item[]::new);

                Solution solve = GA.solve(MaterialsConstant.MASTER_LENGTH, MaterialsConstant.MASTER_WIDTH, items);
                List<PlaceItem> placeItemList = solve.getPlaceItemList();

                workOrder.setSchedulingNum(new BigDecimal(placeItemList.size()));
                workOrder.setUsedArea(BigDecimal.valueOf(solve.getTotal()));
                workOrder.setUseRatio(BigDecimal.valueOf(solve.getUseRatio() * 100));
                workOrder.setMaxLength(BigDecimal.valueOf(solve.getMaxLength()));

                Map<Long, PlaceItem> placeItemMap = placeItemList.stream().collect(Collectors.toMap(PlaceItem::getId, Function.identity()));

                tempWorkOrderDetailList = tempWorkOrderDetailList.stream()
                        .filter(item -> {
                            PlaceItem placeItem = placeItemMap.get(item.getId());
                            if (placeItem == null) {
                                workOrderDetailService.removeById(item.getId());

                                Long orderSkuId = item.getOrderSkuId();
                                OrderSku orderSku = orderSkuService.getById(orderSkuId);
                                orderSku.setSchedulingNum(orderSku.getSchedulingNum().subtract(BigDecimal.ONE));
                                orderSku.setCompleteScheduling(StatusConstant.NO);
                                orderSkuService.updateById(orderSku);

                                return false;
                            }
                            return true;
                        })
                        .peek(item -> {
                            PlaceItem placeItem = placeItemMap.get(item.getId());
                            item.setX(BigDecimal.valueOf(placeItem.getX()));
                            item.setY(BigDecimal.valueOf(placeItem.getY()));
                            item.setLength(BigDecimal.valueOf(placeItem.getLength() - MaterialsConstant.RESERVE));
                            item.setWidth(BigDecimal.valueOf(placeItem.getWidth() - MaterialsConstant.RESERVE));
                            item.setBleedingLength(BigDecimal.valueOf(placeItem.getLength()));
                            item.setBleedingWidth(BigDecimal.valueOf(placeItem.getWidth()));
                            item.setRotate(placeItem.isRotate() ? StatusConstant.YES : StatusConstant.NO);
                        })
                        .collect(Collectors.toList());
            }

            workOrderService.updateById(workOrder);
            workOrderDetailService.updateBatchById(tempWorkOrderDetailList);

        }
    }

}
