package com.sd.business.service.purchase.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sd.business.entity.bom.po.Bom;
import com.sd.business.entity.bom.po.BomClassify;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.purchase.dto.PurchaseBomDetailsSelectDto;
import com.sd.business.entity.purchase.dto.PurchaseBomDto;
import com.sd.business.entity.purchase.dto.PurchaseBomSelectDto;
import com.sd.business.entity.purchase.po.Purchase;
import com.sd.business.entity.purchase.po.PurchaseBom;
import com.sd.business.entity.purchase.vo.PurchaseBomDetailsVo;
import com.sd.business.entity.purchase.vo.PurchaseBomInfoVo;
import com.sd.business.entity.purchase.vo.PurchaseBomSelectVo;
import com.sd.business.entity.purchase.vo.PurchaseBomVo;
import com.sd.business.mapper.purchase.PurchaseBomMapper;
import com.sd.business.service.bom.BomClassifyService;
import com.sd.business.service.purchase.PurchaseBomService;
import com.sd.framework.util.Assert;
import com.sd.framework.util.sql.Sql;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * 采购合同bom 服务实现类
 * </p>
 *
 * @author
 * @since 2023-12-12
 */
@Service
public class PurchaseBomServiceImpl extends ServiceImpl<PurchaseBomMapper, PurchaseBom> implements PurchaseBomService {

    @Autowired
    private BomClassifyService bomClassifyService;

    @Override
    public Page<PurchaseBomSelectVo> getPage(PurchaseBomSelectDto dto) {
        List<Long> childrenIdList = null;
        if (ObjectUtil.isNotNull(dto.getBomClassifyId())) {
            childrenIdList = bomClassifyService.getChildrenIdList(dto.getBomClassifyId());
        }
        if (ObjectUtil.isNotNull(dto.getBomClassifyIdList())) {
            childrenIdList = bomClassifyService.getChildrenIdList(dto.getBomClassifyIdList());
        }

        Page<PurchaseBomSelectVo> page = Sql.create(PurchaseBomSelectVo.class)
                .selectAs(Bom::getId, PurchaseBomSelectVo::getBomId)
                .select(Bom::getCode)
                .select(Bom::getName)
                .select(Bom::getSpecies)
                .from(PurchaseBom.class)
                .innerJoin(BomSpec.class, BomSpec::getId, PurchaseBom::getBomSpecId)
                .innerJoin(Bom.class, Bom::getId, BomSpec::getBomId)
                .eq(PurchaseBom::getPurchaseId, dto.getPurchaseId())
                .eq(Bom::getName, dto.getBomName())
                .like(Bom::getCode, dto.getBomCode())
                .eq(Bom::getSpecies, dto.getSpecies())
                .like(BomSpec::getColour, dto.getColour())
                .like(BomSpec::getCode, dto.getBomSpecCode())
                .like(BomSpec::getName, dto.getBomSpecName())
                .in(Bom::getBomClassifyId, childrenIdList)
                .orderByDesc(PurchaseBom::getId)
                .groupBy(Bom::getId)
                .page(dto);

        List<PurchaseBomSelectVo> records = page.getRecords();
        if (records.isEmpty()) {
            return page;
        }
        List<Long> bomIdList = records.stream().map(PurchaseBomSelectVo::getBomId).collect(Collectors.toList());
        List<PurchaseBomInfoVo> list = Sql.create(PurchaseBomInfoVo.class)
                .selectAll(PurchaseBom.class)
                .selectAs(BomSpec::getId, PurchaseBomInfoVo::getBomSpecId)
                .select(BomSpec::getBomId)
                .select(BomSpec::getName)
                .select(BomSpec::getCode)
                .select(BomSpec::getColour)
                .select(BomSpec::getCostPrice)
                .select(BomSpec::getInternalSellingPrice)
                .select(BomSpec::getExternalSellingPrice)
                .select(BomSpec::getLength)
                .select(BomSpec::getWidth)
                .select(BomSpec::getHeight)
                .select(BomSpec::getNetWeight)
                .from(PurchaseBom.class)
                .innerJoin(BomSpec.class, BomSpec::getId, PurchaseBom::getBomSpecId)
                .eq(PurchaseBom::getPurchaseId, dto.getPurchaseId())
                .in(BomSpec::getBomId, bomIdList)
                .like(BomSpec::getColour, dto.getColour())
                .like(BomSpec::getCode, dto.getBomSpecCode())
                .like(BomSpec::getName, dto.getBomSpecName())
                .orderByDesc(PurchaseBom::getId)
                .list();

        Map<Long, List<PurchaseBomInfoVo>> bomSpecIdMap = list.stream().collect(Collectors.groupingBy(PurchaseBomInfoVo::getBomId));
        for (PurchaseBomSelectVo record : records) {
            record.setBomSpecList(bomSpecIdMap.get(record.getBomId()));
        }

        return page;
    }

    @Override
    public PurchaseBomVo detail(Long id) {

        PurchaseBomVo vo = Sql.create(PurchaseBomVo.class)
                .selectAll(PurchaseBom.class)
                .from(PurchaseBom.class)
                .eq(PurchaseBom::getId, id)
                .one();

        Assert.notNull(vo, "未知数据");

        return vo;
    }

    @Override
    public void add(PurchaseBomDto dto) {
        save(dto);
    }

    @Override
    public void edit(PurchaseBomDto dto) {
        updateById(dto);
    }

    @Override
    public void delete(Long id) {
        removeById(id);
    }

    @Override
    public Page<PurchaseBomDetailsVo> getDetailsPage(PurchaseBomDetailsSelectDto dto) {

        Page<PurchaseBomDetailsVo> page = Sql.create(PurchaseBomDetailsVo.class)
                .selectAll(PurchaseBom.class)
                .selectAs(Purchase::getCode, PurchaseBomDetailsVo::getPurchaseCode)
                .selectAs(BomSpec::getCode, PurchaseBomDetailsVo::getBomSpecCode)
                .selectAs(BomSpec::getName, PurchaseBomDetailsVo::getBomSpecName)
                .select(BomSpec::getLength)
                .select(BomSpec::getWidth)
                .select(BomSpec::getHeight)
                .selectAs("t1.purchase_quantity - t1.arrival_quantity + t1.return_quantity", PurchaseBomDetailsVo::getInTransitQuantity)
                .from(PurchaseBom.class)
                .innerJoin(Purchase.class, Purchase::getId, PurchaseBom::getPurchaseId)
                .innerJoin(BomSpec.class, BomSpec::getId, PurchaseBom::getBomSpecId)
                .like(Purchase::getCode, dto.getPurchaseCode())
                .like(BomSpec::getCode, dto.getBomSpecCode())
                .like(BomSpec::getName, dto.getBomSpecName())
                .orderByDesc(PurchaseBom::getId)
                .page(dto);

        return page;
    }

}
