package com.sd.business.util.code;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.service.IService;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.framework.mybatis.holder.LogicHolder;

import java.util.Date;
import java.util.Map;

public class CodeFactory {

    private final String prefix;
    private final int length;
    private final String codeFieldName;
    private final IService<?> service;
    private Integer number;

    private CodeFactory(CodeEnum codeEnum) {
        this.service = codeEnum.getService();
        this.codeFieldName = codeEnum.getCodeFieldName();
        this.length = codeEnum.getLength();

        if (ObjectUtil.isNotEmpty(codeEnum.getDateFormat())) {
            Date date = new Date();
            String format = DateUtil.format(date, codeEnum.getDateFormat());
            this.prefix = codeEnum.getPrefix() + format;
        } else {
            this.prefix = codeEnum.getPrefix();
        }
    }

    public static CodeFactory create(CodeEnum codeEnum) {
        return new CodeFactory(codeEnum);
    }

    public String next() {
        if (number == null) {

            Object obj;
            Boolean logicHolder = LogicHolder.getLogicHolder();
            LogicHolder.setLogicHolder(true);
            try {
                String column = StrUtil.toUnderlineCase(codeFieldName);
                obj = service.query().likeRight(column, prefix).orderByDesc(column).last("limit 1").one();
            } finally {
                LogicHolder.setLogicHolder(logicHolder);
            }

            if (obj == null) {
                number = 0;
            } else {
                Map<String, Object> map = Convert.toMap(String.class, Object.class, obj);
                String code = Convert.toStr(map.get(StrUtil.toCamelCase(codeFieldName)));
                number = Convert.toInt(code.substring(prefix.length()));

                if (ObjectUtil.isEmpty(number)) {
                    throw new ServiceException("生成系统编码错误，请联系管理员");
                }
            }
        }

        return prefix + String.format("%0" + length + "d", ++number);
    }

}
